﻿package com.niu.ben;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Environment;
import android.text.TextUtils;
import android.util.Log;
import android.widget.Toast;

/**
 * @author niuben
 * @createTime 2015年1月23日
 * @decribe 1.封装常用的log
 *                  2.OPEN_LOGCAT值【true：Logcat显示Log；false：Logcat不显示Log】
 *                  3.OPEN_LOG_TO_FILE【true：Log信息写入文件；false：Log信息不写入文件】
 *                  4.显示toast时同时打印日志
 */
public class LogUtil {
	private static final String  LOG_D="D",//Debug
	LOG_I="I",//Info
	LOG_E="E",//Error
	LOG_V="V",//Verbose
	LOG_W="W",//Warning
	LOG_START="start",
	LOG_END="end",
	LOG_SHOWTOAST="showToast",
	LTAG="LogUtil";
	
	private static boolean OPEN_LOGCAT = true;	//true：Logcat显示Log；false：Logcat不显示Log；
	private static boolean OPEN_LOG_TO_FILE = true;//true：Log信息写入文件；false：Log信息不写入文件
	
    private static String filePath = "";
    
    private static String curDate = "";//当前时间
    private static File curLogFile = null;
    private static BufferedWriter out = null;
    
    /**
     * 设置存储文档的目录
     * @param filePath
     */
    public static void setFilePath(String filePath) {
		LogUtil.filePath = filePath;
	}
    
    private static String getFilePath() {
    	if (TextUtils.isEmpty(LogUtil.filePath)){
    		if (Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_MOUNTED)) {//判断sd卡是否存在
    			filePath= Environment.getExternalStorageDirectory().toString()+File.separator +  "NLOG";
    		}
    	}
    	return filePath;
    }
    
    /**
     * 打开LogCat
     * @param openLogCat
     */
    public static void setOpenLogCat(boolean openLogCat){
    	LogUtil.OPEN_LOGCAT=openLogCat;
    }
    
    public static boolean getOpenLogCat(){
    	return LogUtil.OPEN_LOGCAT;
    }
    
    public static void setOpenLogToFile(boolean openLogToFile){
    	LogUtil.OPEN_LOG_TO_FILE=openLogToFile;
    }
    public static boolean getOpenLogToFile(){
    	return LogUtil.OPEN_LOG_TO_FILE;
    }
	
    public static void d(String msg) {
    	doLog( LOG_D, "", msg);
    }
    
    public static void i(String msg) {
        doLog(LOG_I, "", msg);
    }
    
    public static void e(String msg, Throwable e) {
    	StringBuffer errorMessage = new StringBuffer();
        errorMessage.append("\n" +" #error: " + e.toString() + "");
        for (StackTraceElement ste: e.getStackTrace()) {
        	errorMessage.append("\n    at " + ste.getClassName() + "(" + ste.getFileName() + ":" + ste.getLineNumber()+")");
        }
        doLog(LOG_E, "", msg+errorMessage);
    }
    
    public static void w(String msg) {
        doLog( LOG_W, "", msg);
    }
    
    public static void v(String msg) {
    	 doLog(LOG_V, "", msg);
    }
    
    public static void start(String msg) {
    	 doLog(LOG_I, LOG_START, msg);
    }
    
    public static void start() {
    	 doLog(LOG_I, LOG_START, "");
    }

    public static void end() {
    	 doLog(LOG_I, LOG_END, "");
    }
    
    public static void end(String msg) {
    	 doLog(LOG_I, LOG_END, msg);
    }
    
    public static void showToast(Context context,String msg) {
        doLog( LOG_I, LOG_SHOWTOAST, msg);
        Toast.makeText(context, msg, Toast.LENGTH_SHORT).show();
    }
    
    private static void doLog(String logLevel,String status,String msg) {
    	String[] classMethod = getClassMethod(new Exception());
    	logcat(classMethod[0], classMethod[1], logLevel, status, msg);
		saveLogToFile(classMethod[0], classMethod[1], logLevel, status, msg);
	}
    
    private static void logcat(String tag,String method, String logLevel,String status, String msg){
    	if (!OPEN_LOGCAT){
    		return;
    	}
    	if (LOG_D.equals(logLevel)){
    		Log.d(tag, method +"("+status+")" + msg);
    		return;
    	}
    	if (LOG_I.equals(logLevel)){
    		Log.i(tag, method +"("+status+")" + msg);
    		return;
    	}
    	if (LOG_E.equals(logLevel)){
    		Log.e(tag, method +"("+status+")" + msg);
    		return;
    	}
    	if (LOG_V.equals(logLevel)){
    		Log.v(tag, method +"("+status+")" + msg);
    		return;
    	}
    	if (LOG_W.equals(logLevel)){
    		Log.w(tag, method +"("+status+")" + msg);
    		return;
    	}
    }
        
	  @SuppressLint("SimpleDateFormat")
	private synchronized static void saveLogToFile(String tag,
              String method, String logLevel,String status, String msg) {
		  if (!OPEN_LOG_TO_FILE){
			  Log.d(LTAG,"OPEN_LOG_TO_FILE="+OPEN_LOG_TO_FILE);
			  return;
		  }
		  
		 if (!createLogFolder()){
			 Log.d(LTAG,"createLogFolder失败");
			 return;
		 }
		 
          try {
        	  String date= new SimpleDateFormat("yyyyMMdd HH:mm:ss.SSS").format(new Date());
        	  createLogFile(date.substring(0, 8));
        	    // 日志[20150304 10:53:02.949(10354/10354)-D-CockroachServer-onStartCommand-]PhoneServer is not running
              String logCon = "["+date+
              		"("+android.os.Process.myPid()+"/"+android.os.Process.myTid()+")" +
              		"-"+logLevel+
              		"-"+tag+
              		"-"+method+
              		"-"+status+
              		"]"+msg;
        	  out.newLine();
              out.write(logCon);
              out.flush();
          } catch (Exception e) {
              Log.e(LTAG, "write log file error", e);
          }
      }
	  
	  /**
	   * 创建目录文件夹
	   * @return
	   */
	  private static boolean createLogFolder(){
		  String filePath = getFilePath();
		 
		  if (TextUtils.isEmpty(filePath)) {
			  Log.d(LTAG, "saveLogToFile, filePath为空");
              return false;
          }
		  
          File dest = new File(filePath);
          if (!dest.exists()) {
        	  return dest.mkdirs();
          }
          dest = null;
          return true;
	  }

	  /**
	   * 1天产生一个文件
	   * @param date
	   * @throws Exception
	   */
	  private static void createLogFile(String date) throws Exception{
		  if (TextUtils.isEmpty(curDate)||curDate.equals(date)){
			  curDate=date;
			  curLogFile=null;
			  out=null;
		  }
		  
		  if (curLogFile==null){
			  curLogFile = new File(filePath, "Log" + curDate+ ".txt");
		  }
		  
          if (!curLogFile.exists()) {
        	  curLogFile.createNewFile();
          }
          
          if (out == null) {
        	  out = new BufferedWriter(new OutputStreamWriter(
			          new FileOutputStream(curLogFile, true)));
          }
	  }
	  
	  	/**
		 * @return 打印日志处的类名和方法名
		 */
	    private static String[] getClassMethod(Exception e) {
	        StackTraceElement STE =e.getStackTrace()[2];
	        String className = STE.getClassName();
	        className = className.substring(className.lastIndexOf(".") + 1);
	        String methodName = STE.getMethodName();
	        STE = null;
	        return new String[] { className, methodName };
	    }
}